/** 2008, Digitalis Informatica. All rights reserved.
 *
 * Distribuicao e Gestao de Informatica, Lda.
 * Estrada de Paco de Arcos num.9 - Piso -1
 * 2780-666 Paco de Arcos
 * Telefone: (351) 21 4408990
 * Fax: (351) 21 4408999
 * http://www.digitalis.pt
 */

package pt.digitalis.maven.ormgenerator.hibernate;

import java.util.List;

import org.hibernate.cfg.reveng.DelegatingReverseEngineeringStrategy;
import org.hibernate.cfg.reveng.ReverseEngineeringStrategy;
import org.hibernate.cfg.reveng.TableIdentifier;

/**
 * Hibernate tools reverse engeneering strategy Custom implementation
 *
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a><br/>
 * @created Jan 9, 2009
 */
public class CustomReverseEngineeringStrategy extends DelegatingReverseEngineeringStrategy {

    /**
     * @see org.hibernate.cfg.reveng.DelegatingReverseEngineeringStrategy#foreignKeyToCollectionName(java.lang.String,
     *      org.hibernate.cfg.reveng.TableIdentifier, java.util.List, org.hibernate.cfg.reveng.TableIdentifier,
     *      java.util.List, boolean)
     */
    @SuppressWarnings("unchecked")
    @Override
    public String foreignKeyToCollectionName(String keyname, TableIdentifier fromTable, List fromColumns,
            TableIdentifier referencedTable, List referencedColumns, boolean uniqueReference) {

        String newTableName = referencedTable.getName().toUpperCase();

        if (newTableName.startsWith("T_"))
            newTableName = newTableName.substring(2);

        if (newTableName.startsWith("TB"))
            newTableName = newTableName.replaceFirst("TB", "TABLE_");

        if (!newTableName.equals(referencedTable.getName()))
            referencedTable =
                    new TableIdentifier(referencedTable.getCatalog(), referencedTable.getSchema(), newTableName);

        newTableName = fromTable.getName().toUpperCase();

        if (newTableName.startsWith("T_"))
            newTableName = newTableName.substring(2);

        if (newTableName.startsWith("TB"))
            newTableName = newTableName.replaceFirst("TB", "TABLE_");

        if (!newTableName.equals(fromTable.getName()))
            fromTable =
                    new TableIdentifier(fromTable.getCatalog(), fromTable.getSchema(), newTableName);

        String collectionName =
                super.foreignKeyToCollectionName(keyname, fromTable, fromColumns, referencedTable, referencedColumns,
                        uniqueReference);

        // int pos = collectionName.lastIndexOf(".");
        //
        // collectionName =
        // collectionName.substring(0, pos) + "." + referencedTable.getSchema().toLowerCase() + "."
        // + collectionName.substring(pos + 1);

        return collectionName;
    }

    /**
     * @see org.hibernate.cfg.reveng.DelegatingReverseEngineeringStrategy#foreignKeyToEntityName(java.lang.String,
     *      org.hibernate.cfg.reveng.TableIdentifier, java.util.List, org.hibernate.cfg.reveng.TableIdentifier,
     *      java.util.List, boolean)
     */
    @SuppressWarnings("unchecked")
    @Override
    public String foreignKeyToEntityName(String keyname, TableIdentifier fromTable, List fromColumnNames,
            TableIdentifier referencedTable, List referencedColumnNames, boolean uniqueReference) {

        // The column name must not be translated here. It is the real column name in the data base used for mappinf
        // purposes. The table name however can be translated for more human readable since it is only the name that
        // will be given to the relation.
        String newTableName = referencedTable.getName().toUpperCase();

        if (newTableName.startsWith("T_"))
            newTableName = newTableName.substring(2);

        if (newTableName.startsWith("TB"))
            newTableName = newTableName.replaceFirst("TB", "TABLE_");

        if (!newTableName.equals(referencedTable.getName()))
            referencedTable =
                    new TableIdentifier(referencedTable.getCatalog(), referencedTable.getSchema(), newTableName);

        return super.foreignKeyToEntityName(keyname, fromTable, fromColumnNames, referencedTable,
                referencedColumnNames, uniqueReference);
    }

    /**
     * @param delegate
     */
    public CustomReverseEngineeringStrategy(ReverseEngineeringStrategy delegate) {
        super(delegate);
    }

    /**
     * @see org.hibernate.cfg.reveng.DelegatingReverseEngineeringStrategy#columnToPropertyName(org.hibernate.cfg.reveng.TableIdentifier,
     *      java.lang.String)
     */
    @Override
    public String columnToPropertyName(TableIdentifier table, String column) {

        column = column.toUpperCase();

        if (column.startsWith("CD_"))
            column = column.replaceFirst("CD_", "CODE_");
        else if (column.startsWith("NM_"))
            column = column.replaceFirst("NM_", "NAME_");
        else if (column.startsWith("DS_"))
            column = column.replaceFirst("DS_", "DESC_");
        else if (column.startsWith("HR_"))
            column = column.replaceFirst("HR_", "HOUR_");
        else if (column.startsWith("DT_"))
            column = column.replaceFirst("DT_", "DATE_");
        else if (column.startsWith("NR_"))
            column = column.replaceFirst("NR_", "NUMBER_");

        if (column.startsWith("T_"))
            column = column.substring(2);

        if (column.startsWith("TB"))
            column = column.replaceFirst("TB", "TABLE_");

        return super.columnToPropertyName(table, column);
    }

    /**
     * @see org.hibernate.cfg.reveng.DelegatingReverseEngineeringStrategy#excludeColumn(org.hibernate.cfg.reveng.TableIdentifier,
     *      java.lang.String)
     */
    @Override
    public boolean excludeColumn(TableIdentifier identifier, String columnName) {

        if (columnName.equalsIgnoreCase("REGISTER_ID"))
            return true;
        else
            return super.excludeColumn(identifier, columnName);
    }

    /**
     * @see org.hibernate.cfg.reveng.DelegatingReverseEngineeringStrategy#excludeTable(org.hibernate.cfg.reveng.TableIdentifier)
     */
    @Override
    public boolean excludeTable(TableIdentifier ti) {
        return super.excludeTable(ti);
    }

    /**
     * @see org.hibernate.cfg.reveng.DelegatingReverseEngineeringStrategy#tableToClassName(org.hibernate.cfg.reveng.TableIdentifier)
     */
    @Override
    public String tableToClassName(TableIdentifier tableIdentifier) {

        String newTableName = tableIdentifier.getName().toUpperCase();

        if (newTableName.startsWith("T_"))
            newTableName = newTableName.substring(2);

        if (newTableName.startsWith("TB"))
            newTableName = newTableName.replaceFirst("TB", "TABLE_");

        if (!newTableName.equals(tableIdentifier.getName()))
            tableIdentifier =
                    new TableIdentifier(tableIdentifier.getCatalog(), tableIdentifier.getSchema(), newTableName);

        return super.tableToClassName(tableIdentifier);
    }
}
