package pt.digitalis.maven.ormgenerator.utils;

import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.DependencyResolutionRequiredException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.project.MavenProject;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;
import org.apache.tools.ant.helper.ProjectHelper2;
import org.apache.tools.ant.types.Path;
import org.codehaus.plexus.util.StringUtils;

/**
 * Run an ANT script
 *
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created Nov 12, 2007
 */
public class AntExecutor {

    /**
     * Run an ANT script within the JVM. FIXME: The PostgreSQL JDBC driver is a dependency to this project. Should not
     * be so as it is a choice of the user witch DB to use and it should be him to provide it as a dependency to it's
     * project. I can't pick up the dependencies from the parent project in ANT and this is why it is like this for now
     *
     * @param antFile
     *            the ANT scripts to be executed
     * @param userProperties
     *            the properties to be set for the ANT script
     * @param mavenProject
     *            the maven project in execution
     * @param pluginArtifacts
     *            the plugin maven project artifacts
     * @param logger
     *            the maven plugin logger
     */
    static public void executeAntScript(File antFile, Properties userProperties, List<Artifact> pluginArtifacts,
            MavenProject mavenProject, Log logger)
    {

        File antBaseDir = antFile.getParentFile();
        Project antProject = new Project();
        Target target;

        antProject.init();
        antProject.setBaseDir(antBaseDir);

        ProjectHelper2 projectHelper = new ProjectHelper2();
        projectHelper.parse(antProject, antFile);

        Enumeration<?> keys = userProperties.keys();
        while (keys.hasMoreElements())
        {
            String key = (String) keys.nextElement();
            String value = userProperties.getProperty(key);
            antProject.setUserProperty(key, value);
        }

        Enumeration<?> targets = antProject.getTargets().elements();

        while (targets.hasMoreElements())
        {
            target = (Target) targets.nextElement();
            target.execute();
        }
    }

    /**
     * Builds a classpath from the dependency list of the maven project
     *
     * @param artifacts
     * @param antProject
     * @return a path
     * @throws DependencyResolutionRequiredException
     */
    static public Path getPathFromArtifacts(Set<Artifact> artifacts, Project antProject)
            throws DependencyResolutionRequiredException
    {

        List<Artifact> list = new ArrayList<Artifact>(artifacts);

        return getPathFromArtifacts(list, antProject);
    }

    /**
     * Builds a classpath from the dependency list of the maven project
     *
     * @param artifacts
     * @param antProject
     * @return a path
     * @throws DependencyResolutionRequiredException
     */
    static public Path getPathFromArtifacts(List<Artifact> artifacts, Project antProject)
            throws DependencyResolutionRequiredException
    {

        if (artifacts == null)
            return new Path(antProject);

        List<String> list = new ArrayList<String>(artifacts.size());
        File file;

        for (Artifact artifact: artifacts)
        {
            file = artifact.getFile();

            if (file == null)
                throw new DependencyResolutionRequiredException(artifact);

            list.add(file.getPath());
        }

        Path p = new Path(antProject);
        p.setPath(StringUtils.join(list.iterator(), File.pathSeparator));

        return p;
    }
}
