/** 2008, Digitalis Informatica. All rights reserved.
 *
 * Distribuicao e Gestao de Informatica, Lda.
 * Estrada de Paco de Arcos num.9 - Piso -1
 * 2780-666 Paco de Arcos
 * Telefone: (351) 21 4408990
 * Fax: (351) 21 4408999
 * http://www.digitalis.pt
 */
package pt.digitalis.maven.ormgenerator.utils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import pt.digitalis.maven.ormgenerator.DAO;

/**
 * Finds generated DAO classes
 *
 * @author Pedro Viegas <a href="mailto:pviegas@digitalis.pt">pviegas@digitalis.pt</a>
 * @created Jan 15, 2009
 */
public class DAOFinder {

    /**
     * Searches for all DAOs in the given package
     *
     * @param basePackageDir
     *            the base package directory to start searching from
     * @return a list of DAO objects found
     */
    static public List<DAO> findAllDAOs(String basePackageDir) {

        List<DAO> results = new ArrayList<DAO>();

        for (File pojoDir : getSchemaDirs(basePackageDir)) {
            File[] pojoArr = pojoDir.listFiles(FileUtils.getJavaSourcesFilter());

            for (int i = 0; i < pojoArr.length; i++) {

                if ( pojoArr[i].getName().startsWith("I") && pojoArr[i].getName().endsWith("DAO.java")) {
                    String pojoName = pojoArr[i].getName();
                    pojoName = pojoName.substring(1, pojoName.length() - 8);
                    DAO dao;

                    if (pojoDir.getAbsolutePath().equals(basePackageDir))
                        dao = new DAO("", pojoName);
                    else
                        dao = new DAO(pojoDir.getName(), pojoName);

                    results.add(dao);
                }
            }
        }

        return results;
    }

    /**
     * Searches for a given DAO
     *
     * @param basePackageDir
     *            the base package directory to start searching from
     * @param entityClassName
     *            the DAO entity class name. Uses notation "I<className>DAO" for interface class search
     * @return the DAO object found or null otherwise
     */
    static public DAO findDAO(String basePackageDir, String entityClassName) {

        for (File pojoDir : getSchemaDirs(basePackageDir)) {
            File[] pojoArr = pojoDir.listFiles(FileUtils.getJavaSourcesFilter());

            if (pojoArr != null)
                for (int i = 0; i < pojoArr.length; i++) {

                    if (("I" + entityClassName + "DAO.java").equalsIgnoreCase(pojoArr[i].getName())) {
                        String pojoName = pojoArr[i].getName();
                        pojoName = pojoName.substring(1, pojoName.length() - 8);

                        if (pojoDir.getAbsolutePath().equals(basePackageDir))
                            return new DAO("", pojoName);
                        else
                            return new DAO(pojoDir.getName(), pojoName);
                    }
                }
        }

        return null;
    }

    /**
     * Returns a set of directories to search for java classes - first level directories only
     *
     * @param basePackageDir
     *            the base package to search in
     * @return the list of dir files
     */
    static public List<File> getSchemaDirs(String basePackageDir) {

        List<File> pojoDirs = new ArrayList<File>();
        File mainDir = new File(basePackageDir);
        pojoDirs.add(mainDir);

        // Search for first level sub-dirs (schema dirs)
        for (File schemaDir : mainDir.listFiles(FileUtils.getDirectoryFilter()))
            pojoDirs.add(schemaDir);

        return pojoDirs;
    }

    /**
     * Returns a set of directories to search for java classes - recurrent sub-directories
     *
     * @param basePackageDir
     *            the base package to search in
     * @return the list of dir files
     */
    static public List<File> getSchemaDirsRecursive(String basePackageDir) {

        List<File> pojoDirs = new ArrayList<File>();
        File mainDir = new File(basePackageDir);

        pojoDirs.add(mainDir);

        File[] innerFiles = mainDir.listFiles(FileUtils.getDirectoryFilter());

        // Search for first level sub-dirs (schema dirs)
        if (innerFiles != null) {
            for (File schemaDir : mainDir.listFiles(FileUtils.getDirectoryFilter()))
                pojoDirs.addAll(getSchemaDirsRecursive(schemaDir.getAbsolutePath()));
        }

        return pojoDirs;
    }
}
