package pt.digitalis.maven.ormgenerator.utils;

import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Map;

import freemarker.cache.ClassTemplateLoader;
import freemarker.template.Configuration;
import freemarker.template.DefaultObjectWrapper;
import freemarker.template.Template;
import freemarker.template.TemplateException;

/**
 * Runs an Freemarker script
 * 
 * @author Antnio Silva <a href="mailto:asilva@digitalis.pt">asilva@digitalis.pt</a>
 * @created 2007/11/14
 */
public class FreemarkerExecutor {

    /**
     * Init the Freemarker configuration with the path of the directory where the templates file should be located and
     * the object wrapper
     * 
     * @param templateBaseDir
     *            the directory where the service.ftl are.
     * @param loader
     * @return The freemarker configurations
     * @throws IOException
     *             Thrown when the template directory cannot be read.
     * 
     */
    public static Configuration prepareFreemarker(String templateBaseDir, Class<?> loader) throws IOException {
        Configuration cfg = new Configuration();
        // Specify the data source where the template files come from.
        cfg.setTemplateLoader(new ClassTemplateLoader(loader, templateBaseDir));

        // Specify how templates will see the data model. This is an advanced topic...
        // but just use this:
        cfg.setObjectWrapper(new DefaultObjectWrapper());
        return cfg;
    }

    /**
     * Process a freemarker template merging the giving map with it and returning the generated file to the
     * generatedFilePath
     * 
     * @param cfg
     *            Configuration of the freemarker.
     * @param map
     *            Data map to be merged with the template.
     * @param templateName
     *            The name of the template.
     * @param generatedFilePath
     *            The generated file path.
     * @throws IOException
     *             Exception thrown when the generated file cannot be created or it cannot write in the generated file.
     * @throws TemplateException
     *             Thrown when the template directory cannot be read.
     */
    public static void processFreemarker(Configuration cfg, Map<String, Object> map, String templateName,
            String generatedFilePath) throws IOException, TemplateException {
        Template temp = cfg.getTemplate(templateName);
        /* Merge data model with template */
        Writer out = new FileWriter(generatedFilePath);
        temp.process(map, out);
        out.flush();
    }
}
