package pt.digitalis.maven.ormgenerator;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import pt.digitalis.maven.ormgenerator.utils.DAOFinder;

/**
 * This class should handler the data in the configuration xml and put it
 *
 * @author António Silva <a href="mailto:asilva@digitalis.pt">asilva@digitalis.pt</a>
 * @created Nov 6, 2007
 */
public class ServiceHandler extends DefaultHandler {

    /** The DAO interface classes base package directory */
    private String basePackageDir;

    /**
     * This flag indicates if we are inside a service xml tag
     */
    private boolean inService = false;

    /**
     * This is a temporary variable that holds the service that is being build at the moment
     */
    private Services service;

    /**
     * Holds all the structure of the xml built
     */
    private ServiceCfg serviceCfg = new ServiceCfg();

    /**
     * Default constructor
     *
     * @param basePackage
     */
    public ServiceHandler(String basePackage) {
        this.basePackageDir = basePackage;
    }

    /**
     * Receive notification of the end of an element and should save the information that is recorded in the data object
     * to the main object.
     *
     * @param uri
     *            The Namespace URI, or the empty string if the element has no Namespace URI or if Namespace processing
     *            is not being performed.
     * @param localName
     *            The local name (without prefix), or the empty string if Namespace processing is not being performed.
     * @param qName
     *            The qualified name (with prefix), or the empty string if qualified names are not available.
     * @exception org.xml.sax.SAXException
     *                Any SAX exception, possibly wrapping another exception.
     * @see org.xml.sax.ContentHandler#endElement
     */
    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException {
        if (qName.equals("service")) {
            this.setInService(false);
            this.getServiceCfg().getServices().add(service);
        }
    }

    /**
     * @return the service
     */
    private Services getService() {
        return service;
    }

    /**
     * The Service Configuration built by this class based on the xml this should only be obtained after running parser
     *
     * @return the serviceCfg
     */
    public ServiceCfg getServiceCfg() {
        return serviceCfg;
    }

    /**
     * @return the inService
     */
    private boolean isInService() {
        return inService;
    }

    /**
     * @param inService
     *            the inService to set
     */
    private void setInService(boolean inService) {
        this.inService = inService;
    }

    /**
     * @param service
     *            the service to set
     */
    private void setService(Services service) {
        this.service = service;
    }

    /**
     * Receive notification of the start of an element and starts the transformation of xml tags to the data objects.
     *
     * @param uri
     *            The NameSpace URI, or the empty string if the element has no NameSpace URI or if NameSpace processing
     *            is not being performed.
     * @param localName
     *            The local name (without prefix), or the empty string if NameSpace processing is not being performed.
     * @param qName
     *            The qualified name (with prefix), or the empty string if qualified names are not available.
     * @param attributes
     *            The attributes attached to the element. If there are no attributes, it shall be an empty Attributes
     *            object.
     * @exception org.xml.sax.SAXException
     *                Any SAX exception, possibly wrapping another exception.
     * @see org.xml.sax.ContentHandler#startElement
     */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        if (qName.equals("service")) {
            this.setInService(true);
            Services localServicePkgs = new Services();
            localServicePkgs.setName(attributes.getValue("name"));
            this.setService(localServicePkgs);
        }
        if (this.isInService()) {
            if (qName.equals("uses")) {
                if (attributes.getIndex("dao") != -1) {
                    String daoEntityName = attributes.getValue("dao");
                    DAO dao = DAOFinder.findDAO(basePackageDir, daoEntityName);

                    if (dao != null)
                        this.getService().getLinkedDAOs().add(dao);
                }
                if (attributes.getIndex("service") != -1) {
                    this.getService().getLinkedServices().add(attributes.getValue("service"));
                }
            }
        }
    }
}
